<?php

namespace Seventhqueen\Typer\Core\Theme;

defined( 'ABSPATH' ) || die();

/**
 * Class Compatibility
 * @package Compatibility
 */
final class Compatibility {

	/**
	 * @var
	 */
	public static $instance;

	/**
	 * @return Compatibility
	 */
	public static function get_instance() {
		if ( self::$instance === null ) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	/**
	 * Compatibility constructor.
	 */
	public function __construct() {
		add_filter( 'script_loader_tag', array( $this, 'filter_script_load_tag' ), 10, 2 );
		add_filter( 'typer-required-plugins', [ $this, 'add_more_required_plugins' ] );
	}

	/**
	 * Adds async/defer attributes to enqueued / registered scripts.
	 *
	 * If #12009 lands in WordPress, this function can no-op since it would be handled in core.
	 *
	 * @link https://core.trac.wordpress.org/ticket/12009
	 *
	 * @param string $tag The script tag.
	 * @param string $handle The script handle.
	 *
	 * @return string Script HTML string.
	 */
	public function filter_script_load_tag( $tag, $handle ) {
		foreach ( array( 'async', 'defer' ) as $attr ) {
			if ( ! wp_scripts()->get_data( $handle, $attr ) ) {
				continue;
			}

			// Prevent adding attribute when already added in #12009.
			if ( ! preg_match( ":\s$attr(=|>|\s):", $tag ) ) {
				$tag = preg_replace( ':(?=></script>):', " $attr", $tag, 1 );
			}

			// Only allow async or defer, not both.
			break;
		}

		return $tag;
	}

	public function add_more_required_plugins( $required_plugins ) {
		$required_plugins[] = [
			'name'               => 'Typer Pro Elements',
			'slug'               => 'typer-pro',
			'fallback_version'   => '1.1.0',
			'required'           => false,
			'force_activation'   => false,
			'force_deactivation' => false,
			'external_url'       => '',
			'description'        => 'Gutenberg blocks, Widgets and other cool stuff.',
			'pro'                => true
		];

		$required_plugins[] = [
			'name'               => 'Front User Profile',
			'slug'               => 'front-user-profile',
			'fallback_version'   => '1.1.0',
			'required'           => false,
			'force_activation'   => false,
			'force_deactivation' => false,
			'external_url'       => '',
			'description'        => 'Adds Front-end User profiles.',
			'pro'                => true
		];

		$required_plugins[] = [
			'name'               => 'SQ Comments Media',
			'slug'               => 'sq-comments-media',
			'required'           => false,
			'force_activation'   => false,
			'force_deactivation' => false,
			'external_url'       => '',
			'description'        => 'Allow users to upload images in comments.',
			'pro'                => true
		];

		$required_plugins[] = [
			'name'               => 'SQ Social Share',
			'slug'               => 'sq-social-share',
			'required'           => false,
			'force_activation'   => false,
			'force_deactivation' => false,
			'external_url'       => '',
			'description'        => 'Social sharing for WordPress content.',
			'pro'                => true
		];

		$required_plugins[] = [
			'name'               => 'Sidebar Generator',
			'slug'               => 'sq-sidebar-generator',
			'required'           => false,
			'force_activation'   => false,
			'force_deactivation' => false,
			'external_url'       => '',
			'description'        => 'Generates as many sidebars as you need. Then place them on any page you wish.',
			'pro'                => true
		];

		$required_plugins[] = [
			'name'               => 'Hide Admin Bar',
			'slug'               => 'sq-hide-admin-bar',
			'required'           => false,
			'force_activation'   => false,
			'force_deactivation' => false,
			'external_url'       => '',
			'description'        => 'Hides the admin bar in front-end area.',
			'pro'                => true
		];

		$required_plugins[] = [
			'name'               => 'Jet Popup',
			'slug'               => 'jet-popup',
			'required'           => false,
			'force_activation'   => false,
			'force_deactivation' => false,
			'external_url'       => '',
			'description'        => 'Build awesome popups using Elementor.',
			'pro'                => true
		];

		$required_plugins[] = [
			'name'               => 'Jet Elements',
			'slug'               => 'jet-elements',
			'required'           => false,
			'force_activation'   => false,
			'force_deactivation' => false,
			'external_url'       => '',
			'description'        => 'Provides awesome widgets for Elementor builder.',
			'pro'                => true
		];

		$required_plugins[] = [
			'name'               => 'Jet Tabs',
			'slug'               => 'jet-tabs',
			'required'           => false,
			'force_activation'   => false,
			'force_deactivation' => false,
			'external_url'       => '',
			'description'        => 'Tabs and Accordions for Elementor Builder.',
			'pro'                => true
		];

		$required_plugins[] = [
			'name'               => 'Envato Market - Auto Theme Updates',
			// The plugin name
			'slug'               => 'envato-market',
			// The plugin slug (typically the folder name)
			'source'             => 'https://envato.github.io/wp-envato-market/dist/envato-market.zip',
			// The plugin source
			'required'           => true,
			// If false, the plugin is only 'recommended' instead of required
			'version'            => '2.0.1',
			// E.g. 1.0.0. If set, the active plugin must be this version or higher, otherwise a notice is presented
			'force_activation'   => false,
			// If true, plugin is activated upon theme activation and cannot be deactivated until theme switch
			'force_deactivation' => false,
			// If true, plugin is deactivated upon theme switch, useful for theme-specific plugins
			'external_url'       => '',
			// If set, overrides default API URL and points to an external URL
			'description'        => 'Enables automatic theme updates on your site',
			'pro'                => true
		];

		return $required_plugins;
	}

}

Compatibility::get_instance();
